<?php

namespace app\controllers;

use app\helpers\ActiveForm;
use app\models\Participant;
use app\models\search\ParticipantSearch;
use app\models\UploadPictureForm;
use Yii;
use yii\web\NotFoundHttpException;
use yii\web\Response;
use yii\web\UploadedFile;

/**
 * ParticipantController implements the CRUD actions for Participant model.
 */
class ParticipantController extends MainController
{
    /**
     * Lists all Participant models.
     * @return mixed
     */
    public function actionIndex()
    {
        $searchModel = new ParticipantSearch();
        $dataProvider = $searchModel->search(Yii::$app->request->queryParams);

        return $this->render('index', [
            'searchModel' => $searchModel,
            'dataProvider' => $dataProvider,
        ]);
    }

    /**
     * Displays a single Participant model.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionView($id)
    {
        return $this->render('view', [
            'model' => $this->findModel($id),
        ]);
    }

    /**
     * Creates a new Participant model.
     * If creation is successful, the browser will be redirected to the 'view' page.
     * @return mixed
     */
    public function actionCreate()
    {
        $model = new Participant();

        if (Yii::$app->request->isAjax && $model->load(Yii::$app->request->post())) {
            Yii::$app->response->format = Response::FORMAT_JSON;
            return ActiveForm::validate($model);
        }

        if ($model->load(Yii::$app->request->post()) && $model->save()) {
            try {
                if (!$model->saveImageRightFile(UploadedFile::getInstance($model, 'image_rights_file'))) {
                    Yii::$app->getSession()->setFlash('danger', $model->getFirstError('image_rights_upload'));

                    return $this->redirect(['update', 'id' => $model->id_participant]);
                }

                return $this->redirect(['view', 'id' => $model->id_participant]);
            } catch (\Exception $e) {
                Yii::$app->getSession()->setFlash('danger', $e->getMessage());
            }
        }

        return $this->render('create', [
            'model' => $model,
        ]);
    }

    /**
     * Creates a new Participant model.
     * If creation is successful, the browser will be redirected to the 'view' page.
     * @return mixed
     */
    public function actionCreateBasic()
    {
        $model = new Participant();
        $model->setScenario(Participant::SCENARIO_BASIC);

        if ($model->load(Yii::$app->request->post()) && $model->save()) {
            return $this->redirect(['view', 'id' => $model->id_participant]);
        }

        return $this->render('create-basic', [
            'model' => $model,
        ]);
    }

    public function actionUpdateBasic($id)
    {
        $model = $this->findModel($id);
        $model->setScenario(Participant::SCENARIO_BASIC);

        if ($model->load(Yii::$app->request->post()) && $model->save()) {
            return $this->redirect(['view', 'id' => $model->id_participant]);
        }

        return $this->render('update-basic', [
            'model' => $model,
        ]);
    }

    /**
     * Updates an existing Participant model.
     * If update is successful, the browser will be redirected to the 'view' page.
     * @param integer $id
     * @return mixed
     * @throws \yii\db\Exception
     * @throws \yii\base\InvalidParamException
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionUpdate($id)
    {
        $model = $this->findModel($id);

        if (Yii::$app->request->isAjax && $model->load(Yii::$app->request->post())) {
            Yii::$app->response->format = Response::FORMAT_JSON;
            return ActiveForm::validate($model);
        }

        if ($model->load(Yii::$app->request->post())) {
            $transaction = Yii::$app->db->beginTransaction();

            try {
                if ($model->save() & $model->saveImageRightFile(UploadedFile::getInstance($model, 'image_rights_file'))) {
                    $transaction->commit();

                    return $this->redirect(['view', 'id' => $model->id_participant]);
                }

            } catch (\Exception $e) {
                $transaction->rollBack();

                Yii::$app->getSession()->setFlash('danger', $e->getMessage());
            }
        }

        return $this->render('update', [
            'model' => $model,
        ]);
    }

    /**
     * Deletes an existing Participant model.
     * If deletion is successful, the browser will be redirected to the 'index' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionDelete($id)
    {
        $this->findModel($id)->delete();

        return $this->redirect(['index']);
    }

    public function actionUploadPicture($id)
    {
        $model = new UploadPictureForm(['participant' => $this->findModel($id)]);

        if ($model->load(Yii::$app->request->post())) {
            $model->picture = UploadedFile::getInstance($model, 'picture');
            if ($model->upload()) {
                Yii::$app->getSession()->setFlash('success', Yii::t('app', 'Foto atualizada com sucesso.'));
            }
        }

        return $this->render('upload-picture', [
            'model' => $model,
        ]);
    }

    public function actionPicture($id, $sizeBig = 1)
    {
        $model = new UploadPictureForm(['participant' => $this->findModel($id)]);
        $filePath = $model->filePath((bool)$sizeBig);

        return $filePath ? Yii::$app->response->sendFile($filePath) : '';
    }

    public function actionImageRights($id)
    {
        $model = $this->findModel($id);
        $filePath = $model->imageRightsFilePath();

        return $filePath ? Yii::$app->response->sendFile($filePath) : '';
    }

    public function actionDeletePicture($id)
    {
        $this->findModel($id)->deletePicture();

        return $this->redirect(['participant/upload-picture', 'id' => $id]);
    }

    public function actionCreateLogin($id)
    {
        $model = $this->findModel($id);

        if ($model->createLogin()) {
            Yii::$app->getSession()->setFlash('success', Yii::t('app', 'Password gerada com sucesso e enviada para o email do participante.'));
        } else {
            Yii::$app->getSession()->setFlash('danger', Yii::t('app', 'Ocorreu um erro ao gerar password do participante.'));
        }

        return $this->redirect(['view', 'id' => $model->id_participant]);
    }

    /**
     * Finds the Participant model based on its primary key value.
     * If the model is not found, a 404 HTTP exception will be thrown.
     * @param integer $id
     * @return Participant the loaded model
     * @throws NotFoundHttpException if the model cannot be found
     */
    protected function findModel($id)
    {
        if (($model = Participant::findOne($id)) !== null) {
            return $model;
        }

        throw new NotFoundHttpException(Yii::t('app', 'The requested page does not exist.'));
    }
}
