<?php

namespace app\controllers;

use app\helpers\ActiveForm;
use app\models\ParticipantProfile;
use app\models\PasswordForm;
use app\models\UploadPictureForm;
use Yii;
use yii\data\ArrayDataProvider;
use yii\filters\AccessControl;
use yii\web\ForbiddenHttpException;
use yii\web\NotFoundHttpException;
use yii\web\Response;
use yii\web\UploadedFile;

/**
 * ParticipantProfileController implements the CRUD actions for Participant model.
 */
class ParticipantProfileController extends MainController
{
    /** @var  ParticipantProfile */
    public $participantProfile;

    public function init()
    {
        if (!Yii::$app->user->isGuest) {
            $this->participantProfile = ParticipantProfile::find()->one();
        }

        parent::init();
    }

    public function behaviors()
    {
        return [
            'access' => [
                'class' => AccessControl::className(),
                'only' => ['update', 'upload-picture'],
                'rules' => [
                    [
                        'allow' => $this->participantProfile && $this->participantProfile->canUpdate(),
                        'denyCallback' => function () {
                            throw new
                            ForbiddenHttpException(Yii::t('app', 'Não tem permissão neste momento para aceder a esta opção (para mais informações contacte o secretariado)'));
                        },
                    ],
                ],
            ],
        ];
    }

    /**
     * @return mixed
     * @throws \yii\base\InvalidParamException
     */
    public function actionIndex()
    {
        return $this->render('index', [
            'dataProvider' => new ArrayDataProvider(['allModels' => [$this->participantProfile]]),
        ]);
    }

    /**
     * Displays a single Participant model.
     * @return mixed
     * @throws \yii\base\InvalidParamException
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionView()
    {
        return $this->render('view', [
            'model' => $this->participantProfile,
        ]);
    }

    /**
     * Updates an existing Participant model.
     * If update is successful, the browser will be redirected to the 'view' page.
     * @return mixed
     * @throws \yii\db\Exception
     * @throws \yii\base\InvalidParamException
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionUpdate()
    {
        $model = $this->participantProfile;

        if (Yii::$app->request->isAjax && $model->load(Yii::$app->request->post())) {
            Yii::$app->response->format = Response::FORMAT_JSON;
            return ActiveForm::validate($model);
        }

        if ($model->load(Yii::$app->request->post())) {
            $transaction = Yii::$app->db->beginTransaction();

            try {
                if ($model->save() & $model->saveImageRightFile(UploadedFile::getInstance($model, 'image_rights_file'))) {
                    $transaction->commit();

                    return $this->redirect(['view']);
                }

            } catch (\Exception $e) {
                $transaction->rollBack();

                Yii::$app->getSession()->setFlash('danger', $e->getMessage());
            }
        }

        return $this->render('update', [
            'model' => $model,
        ]);
    }

    /**
     * Updates an existing Participant model.
     * If update is successful, the browser will be redirected to the 'view' page.
     * @return mixed
     * @throws \yii\db\Exception
     * @throws \yii\base\InvalidParamException
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionPassword()
    {
        $model = new PasswordForm($this->participantProfile->login);

        if ($model->load(Yii::$app->request->post())) {
            if ($model->save()) {
                Yii::$app->getSession()->setFlash('success', Yii::t('app', 'Password alterada com sucesso.'));
            } else {
                Yii::$app->getSession()->setFlash('danger', Yii::t('app', 'Erro ao alterar a password.'));
            }
        }

        return $this->render('password', [
            'model' => $model,
        ]);
    }

    public function actionUploadPicture()
    {
        $model = new UploadPictureForm(['participant' => $this->participantProfile]);

        if ($model->load(Yii::$app->request->post())) {
            $model->picture = UploadedFile::getInstance($model, 'picture');
            if ($model->upload()) {
                Yii::$app->getSession()->setFlash('success', Yii::t('app', 'Foto atualizada com sucesso.'));
            }
        }

        return $this->render('upload-picture', [
            'model' => $model,
        ]);
    }

    public function actionPicture($sizeBig = 1)
    {
        $model = new UploadPictureForm(['participant' => $this->participantProfile]);
        $filePath = $model->filePath((bool)$sizeBig);

        return $filePath ? Yii::$app->response->sendFile($filePath) : '';
    }

    public function actionImageRights()
    {
        $model = $this->participantProfile;
        $filePath = $model->imageRightsFilePath();

        return $filePath ? Yii::$app->response->sendFile($filePath) : '';
    }

    public function actionDeletePicture()
    {
        $this->participantProfile->deletePicture();

        return $this->redirect(['upload-picture']);
    }
}
