<?php

namespace app\modules\user\models;

use Yii;
use yii\db\ActiveRecord;
use yii\web\IdentityInterface;
use yii\helpers\Inflector;
use ReflectionClass;

class User extends ActiveRecord implements IdentityInterface
{
    const STATUS_INACTIVE = 0;

    const STATUS_ACTIVE = 1;

    const STATUS_BANNED = 2;
    
    public static $status_s = 
    [
        self::STATUS_ACTIVE => 'Ativo',
        self::STATUS_INACTIVE => 'Aguardando confirmação',
        self::STATUS_BANNED => 'Excluído',
    ];

    public $currentPassword;

    public $newPassword;

    public $newPasswordConfirm;

    protected $permissionCache = [];

    public $module;
    
    public function init()
    {
        if (!$this->module)
        {
            $this->module = Yii::$app->getModule("user");
        }
    }

    public function rules()
    {
        $rules = 
        [
            [['email', 'username'], 'string', 'max' => 255],
            [['email', 'username'], 'unique'],
            [['email', 'username'], 'filter', 'filter' => 'trim'],
            [['email'], 'email'],
            [['newPassword'], 'string', 'min' => 8],
            [['newPassword'], 'filter', 'filter' => 'trim'],
            [['newPassword'], 'required', 'on' => ['register', 'reset']],
            [['newPasswordConfirm'], 'required', 'on' => ['reset']],
            [['newPasswordConfirm'], 'compare', 'compareAttribute' => 'newPassword', 'message' => "Senhas não conferem"],
            [['currentPassword'], 'validateCurrentPassword', 'on' => ['account']],
        ];
        if ($this->password) 
        {
            $rules[] = [['currentPassword'], 'required', 'on' => ['account']];
        }

        if ($this->module->requireEmail) 
        {
            $rules[] = ["email", "required"];
        }
        if ($this->module->requireUsername)
        {
            $rules[] = ["username", "required"];
        }

        return $rules;
    }

    public function validateCurrentPassword()
    {
        if (!$this->validatePassword($this->currentPassword))
        {
            $this->addError("currentPassword", "Current password incorrect");
        }
    }

    public function attributeLabels()
    {
        return [
            'id' => Yii::t('user', 'ID'),
            'role_id' => Yii::t('user', 'Permissão'),
            'status' => Yii::t('user', 'Status'),
            'email' => Yii::t('user', 'Email'),
            'username' => Yii::t('user', 'Username'),
            'password' => "Senha",
            'auth_key' => Yii::t('user', 'Auth Key'),
            'access_token' => Yii::t('user', 'Access Token'),
            'logged_in_ip' => Yii::t('user', 'Logged In Ip'),
            'logged_in_at' => Yii::t('user', 'Logged In At'),
            'created_ip' => Yii::t('user', 'Created Ip'),
            'created_at' => Yii::t('user', 'Created At'),
            'updated_at' => Yii::t('user', 'Updated At'),
            'banned_at' => Yii::t('user', 'Banned At'),
            'banned_reason' => Yii::t('user', 'Banned Reason'),
            'currentPassword' => Yii::t('user', 'Current Password'),
            'newPassword' => "Senha",
            'newPasswordConfirm' => "Confirme sua senha",
        ];
    }
    
    public function behaviors() 
    {
        $behaviors = 
        [
            'timestamp' => 
            [
                'class' => 'yii\behaviors\TimestampBehavior',
                'value' => function ($event) 
                {
                    return gmdate("Y-m-d H:i:s");
                },
            ],
        ];

        return array_merge(parent::behaviors(), $behaviors);
    }

    public function getProfile()
    {
        $profile = $this->module->model("Profile");
        return $this->hasOne($profile::className(), ['user_id' => 'id']);
    }

    public function getRole()
    {
        $role = $this->module->model("Role");
        return $this->hasOne($role::className(), ['id' => 'role_id']);
    }

    public function getUserTokens()
    {
        $userToken = $this->module->model("UserToken");
        return $this->hasMany($userToken::className(), ['user_id' => 'id']);
    }

    public function getUserAuths()
    {
        return $this->hasMany(UserAuth::className(), ['user_id' => 'id']);
    }

    public static function findIdentity($id)
    {
        return static::findOne($id);
    }

    public static function findIdentityByAccessToken($token, $type = null)
    {
        return static::findOne(["access_token" => $token]);
    }

    public function getId()
    {
        return $this->id;
    }

    public function getAuthKey()
    {
        return $this->auth_key;
    }

    public function validateAuthKey($authKey)
    {
        return $this->auth_key === $authKey;
    }

    public function validatePassword($password)
    {
        return Yii::$app->security->validatePassword($password, $this->password);
    }

    public function beforeSave($insert)
    {
        $dirtyAttributes = $this->getDirtyAttributes();
        
        if (isset($dirtyAttributes["password"])) 
        {
            $this->newPassword = $dirtyAttributes["password"];
        }

        if ($this->newPassword)
        {
            $this->password = Yii::$app->security->generatePasswordHash($this->newPassword);
        }

        if ($this->banned_at) 
        {
            $this->banned_at = gmdate("Y-m-d H:i:s");
        }
        
        if (!$this->username) 
        {
            $parts = explode('@', $this->email);
            $this->username = $parts[0] . mt_rand(0, 99)  . mt_rand(0, 99)  . mt_rand(0, 99);
        }

        $nullAttributes = ["email", "username", "banned_at", "banned_reason"];
        foreach ($nullAttributes as $nullAttribute)
        {
            $this->$nullAttribute = $this->$nullAttribute ? $this->$nullAttribute : null;
        }

        return parent::beforeSave($insert);
    }

    public function setRegisterAttributes($roleId, $status = null)
    {
        $attributes = 
        [
            "role_id" => $roleId,
            "created_ip" => (isset(Yii::$app->request->userIP)) ? Yii::$app->request->userIP : null,
            "auth_key" => Yii::$app->security->generateRandomString(),
            "access_token" => Yii::$app->security->generateRandomString(),
            "status" => static::STATUS_ACTIVE,
        ];

        $emailConfirmation = $this->module->emailConfirmation;
        $requireEmail = $this->module->requireEmail;
        $useEmail = $this->module->useEmail;
        if ($status)
        {
            $attributes["status"] = $status;
        }
        elseif ($emailConfirmation && $requireEmail) 
        {
            $attributes["status"] = static::STATUS_INACTIVE;
        } 

        $this->setAttributes($attributes, false);
        return $this;
    }

    public function checkEmailChange()
    {
        if ($this->email == $this->getOldAttribute("email")) 
        {
            return false;
        }

        if (!$this->module->emailChangeConfirmation) 
        {
            return false;
        }

        if (!$this->email) 
        {
            return false;
        }

        $newEmail = $this->email;
        $this->status = static::STATUS_BANNED;
        $this->email = $this->getOldAttribute("email");
        return $newEmail;
    }

    public function updateLoginMeta()
    {
        $this->logged_in_ip = Yii::$app->request->userIP;
        $this->logged_in_at = gmdate("Y-m-d H:i:s");
        return $this->save(false, ["logged_in_ip", "logged_in_at"]);
    }
       
    public function confirm($newEmail)
    {
        $this->status = static::STATUS_ACTIVE;

        $success = true;
        if ($newEmail) 
        {
            $checkUser = static::findOne(["email" => $newEmail]);
            if ($checkUser) 
            {
                $success = false;
            } 
            else
            {
                $this->email = $newEmail;
            }
        }

        $this->save(false, ["email", "status"]);
        return $success;
    }

    public function can($permissionName, $params = [], $allowCaching = true)
    {
        $auth = Yii::$app->getAuthManager();
        if ($auth) 
        {
            if ($allowCaching && empty($params) && isset($this->permissionCache[$permissionName])) 
            {
                return $this->permissionCache[$permissionName];
            }
            $access = $auth->checkAccess($this->getId(), $permissionName, $params);
            if ($allowCaching && empty($params)) 
            {
                $this->permissionCache[$permissionName] = $access;
            }
            return $access;
        }

        return $this->role->checkPermission($permissionName);
    }

    public function getDisplayName()
    {
        return $this->username ?: $this->email ?: $this->id;
    }

    public function sendEmailConfirmation($userToken)
    {
        $mailer = Yii::$app->mailer;
        $oldViewPath = $mailer->viewPath;
        $mailer->viewPath = $this->module->emailViewPath;

        $user = $this;
        $profile = $user->profile;
        $email = $userToken->data ?: $user->email;
        $subject = "Confirmação de Usuário";
        $result = $mailer->compose('confirmEmail', compact("subject", "user", "profile", "userToken"))
            ->setTo($email)
            ->setSubject($subject  . ' - Lafs Contabilidade')
            ->send();

        $mailer->viewPath = $oldViewPath;
        return $result;
    }
    
    public function sendEmailRegister($userToken)
    {
        $mailer = Yii::$app->mailer;
        $oldViewPath = $mailer->viewPath;
        $mailer->viewPath = $this->module->emailViewPath;

        $user = $this;
        $profile = $user->profile;
        $email = $userToken->data ?: $user->email;
        $subject = "Cadastro de Usuário";
        $result = $mailer->compose('registerEmail', compact("subject", "user", "profile", "userToken"))
            ->setTo($email)
            ->setSubject($subject  . ' - Lafs Contabilidade')
            ->send();

        $mailer->viewPath = $oldViewPath;
        return $result;
    }
    
    public static function statusDropdown()
    {
        static $dropdown;
        $constPrefix = "STATUS_";
        if ($dropdown === null) 
        {
            $reflClass = new ReflectionClass(get_called_class());
            $constants = $reflClass->getConstants();

            foreach ($constants as $constantName => $constantValue) 
            {
                if (strpos($constantName, $constPrefix) === 0) 
                {
                    $prettyName = str_replace($constPrefix, "", $constantName);
                    $prettyName = Inflector::humanize(strtolower($prettyName));
                    $dropdown[$constantValue] = $prettyName;
                }
            }
        }

        return $dropdown;
    }
}